# This is for analysing DISSECT results on olave data.
#  Forked from DISSECT_9DOmegaEtc.r on 2014-07-22. 

library(ape)
library(phangorn)

################################################################################
###################### Setting up file paths, loading code #####################

# Where this code is
setwd("C:/Users/Work/AAA/Programming/biology/STACEYOlaveSimulations")
 

################################################################################
################# The directory containing output to be analysed ###############

ALLRESULTS.DIR <- "C:/Users/Work/AAA/Programming/ProgramOutput/Olave-STACEY/"

################################################################################
######################### Top level functions ##################################

# Implementation notes.
# Table of output from STACEY is called Omega.
# The clusterings occur in table rows. Some functions take table rows as 
# arguments, others just the columns corresponding to clusterings.
# The table is augmented with an extra column, making two kinds of table row.


# extracts a clustering from a table row or augmented table row
clustering.from.row <- function(Z) {
augmented <- (length(grep("smdist", names(Z))) > 0)
z <- as.vector(t(Z[-(1:4)]))
if (augmented) {
  z <- z[-length(z)]
  }
z
}



# Makes the similarity matrix for the whole table.
# This is done pair by pair for all rows.
simmatrix.from.clusterings <- function(Omega) {
  # Minimal cluster names are column names omitting first 4 
  mincl.names <- colnames(Omega)[-(1:4)]
  # Make the similarity matrix  
  nmincls <- length(mincl.names)
  sim <- matrix(0, ncol=nmincls, nrow=nmincls, dimnames=list(mincl.names, mincl.names))
  for (i in 1:nmincls) {
    for (j in 1:nmincls) {
      coli <- Omega[,mincl.names[i]]
      colj <- Omega[,mincl.names[j]]
      w <- (coli == colj)
      sim[i,j] <- sum(Omega[w,"fraction"])
    }
  }
  sim
}

# Makes a similarity matrix for clustering indexed by r 
single.simmatrix.from.clusterings <- function(Omega.or.AugOmega, r) {  
  zr <- clustering.from.row(Omega.or.AugOmega[r, ])
  com.clustering(zr)
}



# makes a vector of distances from clusterings to the mean simmat
dists.to.posterior <- function(Omega) {
  nclusts <- nrow(Omega)
  dists <- rep(0, nclusts)
  SM <- simmatrix.from.clusterings(Omega)
  for (m in 1:nclusts) {
    Cm <- com.table.row(Omega[m,])
    dists[m] <- simmatrixdist(Cm, SM)
    }
  dists
}

# adds a column of distances to table and returns table sorted on these 
augment.clusterings <- function(Omega) {
  dists <- dists.to.posterior(Omega)
  AugOmega <- cbind(Omega, dists)
  colnames(AugOmega) <- c(colnames(Omega), "smdist")
  st <- sort(AugOmega[,"smdist"], index.return=TRUE)
  AugOmega[st$ix,]
}


# finds the boundary of a credible set.  
credible.boundary <- function(AugOmega, size) {
  nclusts <- nrow(AugOmega)
  cum.p <- 0
  bdy.dist <- -1
  cum.p.in <- -1
  cum.p.out <- -1
  for (m in 1:nclusts) {
    pm <- AugOmega[m,"fraction"]
    new.cum.p <- cum.p + pm 
    if (cum.p < size  &&  new.cum.p >= size) {
      bdy.dist <- AugOmega[m,"smdist"]
      cum.p.in <- cum.p
      cum.p.out <- new.cum.p 
      }
    cum.p <- new.cum.p
    }
  stopifnot(bdy.dist >= 0)
  list(bdy.dist=bdy.dist, cum.p.in=cum.p.in, cum.p.out=cum.p.out, size=size)
}


# returns a membership value (1 in, 0 out) for clustering z
within.credible.set <- function(z, simmatrix, cred.bdy) {
  zcom <- com.clustering(z)
  zdist <- simmatrixdist(simmatrix, zcom)
  cpin <- cred.bdy$cum.p.in
  cpout <- cred.bdy$cum.p.out
  size <- cred.bdy$size
  creddist <- cred.bdy$bdy.dist
  if (zdist < creddist) {
    return (1)
    }
  if (zdist > creddist) {
    return (0)
    }
  return ((size - cpin) / (cpout - cpin) > 0.5)
}


# makes a matrix from a clustering
com.clustering <- function(z) {
  n <- length(z)
  Com <- matrix(0, n, n)
  for (i in 1:n) {
    Com[i,] <- (z[i]==z)
    }
  Com
}


# makes a matrix from a row in table 
com.table.row <- function(Z) {
  com.clustering(clustering.from.row(Z))
}


# compares the clustering in a row in table Z to a true clustering
table.row.correct <- function(Z, tru) {
  nc <- Z[4]
  z <- clustering.from.row(Z)
  for (zi in 1:nc) {
    wzi <- which(z == zi)
    trui <- tru[wzi[1]]
    wtrui <- which(tru == trui)
    if (length(wzi) != length(wtrui)) {
      return (FALSE)
      }
    if (!all(wzi==wtrui)) {
      return (FALSE)
      }      
    }
  return (TRUE)
}

# mean absolute difference between two matrices 
# (in the context of this script, the diagonal are always equal)
simmatrixdist <- function(M1, M2)   {
  n <- ncol(M1)
  sum(abs(M1-M2))/(n*(n-1))
}


# compares the clustering in a row in table Z to a true clustering 
# and returns the I_s value defined by Olave et al.
olave.Is.score <- function(Z, tru) {
  Z <- as.vector(Z)
  tru <- as.vector(tru)
  Zids <- unique(Z)
  Tids <- unique(tru)
  kr <- length(Tids)
  x <- 0
  for (id in Tids) {
    species.i <- which(tru == id) # species.i is true species
    nri <- length(species.i)
    ngi <- length(unique(Z[species.i]))
    x <- x + (ngi-1)/(nri-1)
  }
x / kr
}


# compares the clustering in a row in table Z to a true clustering 
# and returns (my own) measure of joins/mis-assignments. 
join.misassign.score <- function(Z, tru) {

  Z <- as.vector(Z)
  tru <- as.vector(tru)
  Zids <- unique(Z)
  Tids <- unique(tru)
  x <- 0
  for (id in Zids) {   
    species.i <- which(Z == id)  # species.i is estimated species
    ngi <- length(unique(tru[species.i]))
    x <- x + (ngi-1)
  }
  x
}




################################################################################
######################## true clusterings and matrices #########################

true.clustering.Olave <- function() {
  sim <- rep(0, 40)
  for (i in 1:40) {
    sim[i] <- floor((i-1)/5)
  }  
sim
}


true.simmatrix.Olave <- function() {
  sim <- matrix(0, ncol=40, nrow=40)
  for (i in 1:40) {
    for (j in 1:40) {
      trupairsame <- (floor((i-1)/5) == floor((j-1)/5))
      if (trupairsame) {
        sim[i,j] <- 1
        }
     }
  }  
sim
}


################################################################################
################################### output #####################################


cat.array <- function(x, firstrep, lastrep) {
  cat(" <- c(")
  for (i in firstrep:lastrep) {
    cat(x[i], sep="") 
    if (i < length(x)) {
      cat(",")
    }
  }
  cat(")\n")
}


# for cat.all.arrays() for output 
results.varname <- function(subdir, i, nam) {
  paste0(subdir, ".", nam)
  }

# output results as R code
cat.all.arrays <- function(arrays) {
  # these are pasted into plotrand.r in latex article directory
  cat(results.varname(arrays$subdir, i, "credible.set.counts"), sep="")
  cat.array(arrays$credible.set.counts, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "mean.simm.dists"), sep="")
  cat.array(arrays$mean.simm.dists, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "randdists.to.maxpp"), sep="")
  cat.array(arrays$randdists.to.maxpp, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "randdists.to.mindist"), sep="")
  cat.array(arrays$randdists.to.mindist, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "ptrue.clustering"), sep="")
  cat.array(arrays$ptrue.clustering, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "olave.Is"), sep="")
  cat.array(arrays$olave.Is, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "max.false.pp"), sep="")
  cat.array(arrays$max.false.pp, arrays$firstrep, arrays$lastrep)
  
  cat(results.varname(arrays$subdir, i, "join.misassign.score"), sep="")
  cat.array(arrays$join.misassign.score, arrays$firstrep, arrays$lastrep)
  
  cat("\n")
}



make.arrays <- function(subdir, firstrep, lastrep) {  
  RESULTS.DIR <- paste0(ALLRESULTS.DIR, subdir)
  
  true.simmatrix <- true.simmatrix.Olave()
  true.clustering <- true.clustering.Olave()
  
  credible.set.counts <- rep(0, lastrep) 
  mean.simm.dists <- rep(0, lastrep)
  randdists.to.maxpp <- rep(0, lastrep)
  randdists.to.mindist <- rep(0, lastrep)
  ptrue.clustering <- rep(0, lastrep) 
  olave.Is <- rep(0, lastrep) 
  max.false.pp <- rep(0, lastrep)
  join.misassign.score <- rep(0, lastrep)
  
  
  for (r in firstrep:lastrep) {
    cat("r", r, "\n")
    Omega <- read.table(paste0(RESULTS.DIR, "/", "6CLU", "r", r, ".txt"), header=TRUE)
    
    AugOmega <- augment.clusterings(Omega)
    cred.bdy <- credible.boundary(AugOmega, .95)
    
    simmatrix <- simmatrix.from.clusterings(Omega)
    mean.simm.dists[r] <- simmatrixdist(true.simmatrix, simmatrix)
    
    maxpp.sim <- single.simmatrix.from.clusterings(Omega, 1)
    
    mindist.sim <- single.simmatrix.from.clusterings(AugOmega, 1)
    randdists.to.maxpp[r] <- simmatrixdist(true.simmatrix, maxpp.sim)
    randdists.to.mindist[r] <- simmatrixdist(true.simmatrix, mindist.sim)
    got <- FALSE
    ptrue <- 0
    for (cc in 1: nrow(Omega)) {
      pp <- Omega[cc, "fraction"]
      correct <- table.row.correct(as.vector(t(Omega[cc,])), true.clustering) 
      if (correct) {
        got <- TRUE
        ptrue <- pp
      }
    }
    ptrue.clustering[r] <- ptrue
    
    if (got) {
      credible.set.counts[r] <- within.credible.set(true.clustering, simmatrix, cred.bdy)
    } else {
      credible.set.counts[r] <- 0
    }
    
    olave.Is[r] <- olave.Is.score(as.vector(t(Omega[1,-(1:4)])), true.clustering)
    
    for (cc in 1: nrow(Omega)) {
      pp <- Omega[cc, "fraction"]
      correct <- table.row.correct(as.vector(t(Omega[cc,])), true.clustering) 
      if (correct) {
        got <- TRUE
        ptrue <- pp
      }
    }  
    
    if (simmatrixdist(true.simmatrix, maxpp.sim) > 0) {
      max.false.pp[r] <- Omega[1, "fraction"]
    }
    
    
    join.misassign.score(as.vector(t(Omega[1,-(1:4)])), true.clustering)
    
    
  }  
  list(subdir=subdir,
       firstrep=firstrep,
       lastrep=lastrep,
       credible.set.counts=credible.set.counts,
       mean.simm.dists=mean.simm.dists,
       randdists.to.maxpp=randdists.to.maxpp,
       randdists.to.mindist=randdists.to.mindist,
       ptrue.clustering=ptrue.clustering, 
       olave.Is=olave.Is,
       max.false.pp=max.false.pp,
       join.misassign.score=join.misassign.score
       )
}



sink.allarrays <- function(subdir, firstrep, lastrep, fname, first) {
  arrays <- make.arrays(subdir, firstrep, lastrep)
  sink(fname, append=!first)
  cat.all.arrays(arrays)
  sink(NULL) 
}



sink.allarrays(subdir="YHg4", firstrep=1, lastrep=50, "Olave-arrays.r", TRUE)
sink.allarrays(subdir="ZHg4", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)

sink.allarrays(subdir="YHg8", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)
sink.allarrays(subdir="ZHg8", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)

sink.allarrays(subdir="YHg14", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)
sink.allarrays(subdir="ZHg14", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)


sink.allarrays(subdir="YEg4", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)
sink.allarrays(subdir="ZEg4", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)

sink.allarrays(subdir="YEg8", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)
sink.allarrays(subdir="ZEg8", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)

sink.allarrays(subdir="YEg14", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)
sink.allarrays(subdir="ZEg14", firstrep=1, lastrep=50, "Olave-arrays.r", FALSE)

