library(MASS)
library(ape)
library(coda)


##########################################################################
##########################################################################

### TOPOLOGY AND NODE HEIGHTS


fpathbaseQ <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/Q/Qg1t10/Q4STSg1t10r1s42"
BIRTHRATE <- 100


article.dpath <- "C:/Users/Work/AAA/Docs/Latex/Biology/STACEY/PopModelThreeMoves/"
article.pdf <- paste0(article.dpath, "topolclustersizesBRnodeheights17BR.pdf")
article.eps <- paste0(article.dpath, "topolclustersizesBRnodeheights17BR.eps")



### NODE HEIGHTS ################################

# theoretical height of k'th speciation in tree under Yule model,
# with n tips, from Stadler, "The conditioned reconstructed process".
# BIRTHRATE is lambda. First one is root.
node.height.density <- function(x, n, k) {
  x <- x * BIRTHRATE
  y <- k + 1
  y <- y * choose(n, k+1)
  y <- y * exp(-(k+1)*x)
  y <- y * (1 - exp(-x))^(n-k-1)
  y * BIRTHRATE
}

# find all heights using branching.times() [which seems very slow btw]
# and sort them.
read.trees.remove.burnin <- function(fpath) {
  alltrees <- read.nexus(file=fpath)
  nalltrees <- length(alltrees)
  burnin <- as.integer(nalltrees/10)
  trees <- alltrees[burnin:nalltrees]  
  trees
}



# find all heights using branching.times() [which seems very slow btw]
# and sort them.
get.node.heights <- function(fpath) {
  trees <- read.trees.remove.burnin(fpath) 
  ntrees <- length(trees)
  bts <- matrix(0, ncol=trees[[1]]$Nnode, nrow=ntrees)
  for (t in 1:ntrees) {
    if (t %% 100 == 0) cat(t, " ")
    if (t %% 1000 == 0) cat("\n")
    bts[t,] <- sort(branching.times(trees[[t]]), decreasing=TRUE)
  }
  bts
}


# plot histogram and true density for one set of operators
plot.hists <- function(bts, ops) {
  nplots <- ncol(bts)
  xx <- seq(from=0, to=5/BIRTHRATE, by=.01/BIRTHRATE)
  ncol <- as.integer(round(sqrt(nplots)))
  nrow <- as.integer(ceiling(nplots/ncol))
  layout(mat<-matrix(c(1:nplots, rep(0,ncol*nrow-nplots)), 
                     nrow=nrow, ncol=ncol, byrow=TRUE))
  for (k in 1:nplots) {
    truehist(bts[,k], 
             main=paste0("Operators ", ops), 
             xlab=paste0("height of speciation ", k),
             ylab="density",
             col="grey")
    lines(x=xx, y=node.height.density(xx, nplots+1, k))
  }
}


get.and.plot.node.heights.for.ops <- function(ops) {
  bts <- get.node.heights(paste0(fpathbaseQ, ops, ".txt"))
  plot.hists(bts, ops)
}


get.and.make.pdf.node.heights.for.ops <- function(ops) {
  pdf(file=paste0(fpathbaseQ, "heights", ops, ".pdf"), width=7, height=9)
  get.and.plot.node.heights.for.ops(ops)
  dev.off()
}



# plot histogram and true density for one set of operators, one speciation
get.and.plot.node.heights.for.ops.kth.speciation <- function(ops, k) {
  bts <- get.node.heights(paste0(fpathbaseQ, ops, ".txt"))
  nplots <- ncol(bts)
  xx <- seq(from=0, to=5/BIRTHRATE, by=.01/BIRTHRATE)
  truehist(bts[,k], 
           main=paste0("Operators ", ops), 
           xlab=paste0("height of speciation ", k),
           ylab="density",
           col="grey")
  lines(x=xx, y=node.height.density(xx, nplots+1, k))
}


get.and.plot.node.heights.for.ops.kth.speciation.article <- function(ops, k, label) {
  bts <- get.node.heights(paste0(fpathbaseQ, ops, ".txt"))
  nplots <- ncol(bts)
  xx <- seq(from=0, to=5/BIRTHRATE, by=.01/BIRTHRATE)
  truehist(bts[,k], 
           main=label, 
           xlab=paste0("height of speciation ", k),
           ylab="density",
           col="grey")
  truedens <- node.height.density(xx, nplots+1, k)
  lines(x=xx, y=truedens)
}






### TOPOLOGY ####################################################

nodeinfo.to.topology4 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 4)
  if (nodeinfo[node,3] == 2) {
    return (paste0(topol, "42"))
  } else {
    return (paste0(topol, "41"))
  }
  stopifnot(FALSE)
}



nodeinfo.to.topology44 <- function(nodeinfo, node1, node2, topol) {
  stopifnot(nodeinfo[node1,5] == 4)
  stopifnot(nodeinfo[node2,5] == 4)
  if (nodeinfo[node1,3] == 2) {
    if (nodeinfo[node2,3] == 2) {
      return (paste0(topol, "42+42"))
    } else {
      return (paste0(topol, "41+42"))
    }
  } else {
    if (nodeinfo[node2,3] == 2) {
      return (paste0(topol, "41+42"))
    } else {
      return (paste0(topol, "41+41"))
    }
  }  
  
  
  topol1 <- nodeinfo.to.topology4(nodeinfo, node1, topol)
  topol2 <- nodeinfo.to.topology4(nodeinfo, node2, topol)
  if (topol1 == topol2) {
    
  }
  if (nodeinfo[node1,3] == 1) {
    topol2 <- nodeinfo.to.topology4(nodeinfo, node2, topol)
    topol <- paste0(topol, "+")
    topol <- nodeinfo.to.topology4(nodeinfo, node1, topol)
  } else {
    topol <- nodeinfo.to.topology4(nodeinfo, node1, topol)
    topol <- paste0(topol, "+")
    topol <- nodeinfo.to.topology4(nodeinfo, node2, topol)    
  }
  return (topol)
}



nodeinfo.to.topology5 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 5)
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "51,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "52")
    return (topol)
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "52")
    return (topol)
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "51,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}

nodeinfo.to.topology6 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 6)
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "61,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "62,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "63")
    return (topol)
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "62,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "61,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}


nodeinfo.to.topology7 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 7)
  if (nodeinfo[node,3] == 6) {
    topol <- paste0(topol, "71,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "72,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "73,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "73,")
    return (nodeinfo.to.topology4(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "72,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "71,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}




nodeinfo.to.topology8 <- function(nodeinfo, node, topol) {
  stopifnot(nodeinfo[node,5] == 8)
  if (nodeinfo[node,3] == 7) {
    topol <- paste0(topol, "81,")
    return (nodeinfo.to.topology7(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 6) {
    topol <- paste0(topol, "82,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 5) {
    topol <- paste0(topol, "83,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,1], topol))
  }
  if (nodeinfo[node,3] == 4) {
    topol <- paste0(topol, "84,")
    return (nodeinfo.to.topology44(nodeinfo, nodeinfo[node,1], nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 3) {
    topol <- paste0(topol, "83,")
    return (nodeinfo.to.topology5(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 2) {
    topol <- paste0(topol, "82,")
    return (nodeinfo.to.topology6(nodeinfo, nodeinfo[node,2], topol))
  }
  if (nodeinfo[node,3] == 1) {
    topol <- paste0(topol, "81,")
    return (nodeinfo.to.topology7(nodeinfo, nodeinfo[node,2], topol))
  }
  stopifnot(FALSE)
}



poedges.to.nodeinfo <- function(po.edge, inode.count) {
  edge.count <- nrow(po.edge)
  tip.count <- inode.count + 1
  nodeinfo <- matrix(0, nrow=edge.count+1, ncol=5)
  # cols are lft, rgt, lftsize, rgtsize, size
  for (n in 1:(edge.count+1)) {
    if (n <= edge.count) {
      i <- po.edge[n,2]
    } else {
      i <- po.edge[edge.count, 1]
    }
    
    
    if (i <= tip.count) {
      nodeinfo[n,1] <- -1
      nodeinfo[n,2] <- -1
      nodeinfo[n,3] <- 0
      nodeinfo[n,4] <- 0
      nodeinfo[n,5] <- 1
    } else {
      children <- which(po.edge[,1] == i)
      stopifnot(length(children)==2)
      nodeinfo[n,1] <- children[1]
      nodeinfo[n,2] <- children[2]
      nodeinfo[n,3] <- nodeinfo[ children[1], 5 ]
      nodeinfo[n,4] <- nodeinfo[ children[2], 5 ]
      nodeinfo[n,5] <- nodeinfo[n,3] + nodeinfo[n,4]
    }
  } 
  nodeinfo
  
}



eighttip.topologies <- matrix(c(
  c("81,71,61,51,41", "(*,(*,(*,(*,U))))", (2/7)*(2/6)*(2/5)*(2/4)*(2/3)),
  c("81,71,61,51,42", "(*,(*,(*,(*,B))))", (2/7)*(2/6)*(2/5)*(2/4)*(1/3)),
  c("81,71,61,52",    "(*,(*,(*,(C,T))))", (2/7)*(2/6)*(2/5)*(2/4)),
  c("81,71,62,41",    "(*,(*,(C,U)))",     (2/7)*(2/6)*(2/5)*(2/3)),
  c("81,71,62,42",    "(*,(*,(C,B)))",     (2/7)*(2/6)*(2/5)*(1/3)),
  c("81,71,63",       "(*,(*,(T,T)))",     (2/7)*(2/6)*(1/5)),
  c("81,72,51,41",    "(*,(C,(*,U)))",     (2/7)*(2/6)*(2/4)*(2/3)),
  c("81,72,51,42",    "(*,(C,(*,B)))",     (2/7)*(2/6)*(2/4)*(1/3)),
  c("81,72,52",       "(*,(C,(C,T)))",     (2/7)*(2/6)*(2/4)),
  c("81,73,41",       "(*,(T,U))",         (2/7)*(2/6)*(2/3)),
  c("81,73,42",       "(*,(T,B))",         (2/7)*(2/6)*(1/3)),
  c("82,61,51,41",    "(C,(*,(*,U)))",     (2/7)*(2/5)*(2/4)*(2/3)),
  c("82,61,51,42",    "(C,(*,(*,B)))",     (2/7)*(2/5)*(2/4)*(1/3)),
  c("82,61,52",       "(C,(*,(C,T)))",     (2/7)*(2/5)*(2/4)),
  c("82,62,41",       "(C,(C,U))",         (2/7)*(2/5)*(2/3)),
  c("82,62,42",       "(C,(C,B))",         (2/7)*(2/5)*(1/3)),
  c("82,63",          "(C,(T,T))",         (2/7)*(1/5)),
  c("83,51,41",       "(T,(*,U))",         (2/7)*(2/4)*(2/3)),
  c("83,51,42",       "(T,(*,B))",         (2/7)*(2/4)*(1/3)),
  c("83,52",          "(T,(C,T))",         (2/7)*(2/4)),
  c("84,41+41",       "(U,U)",             (1/7)*(2/3)*(2/3)),
  c("84,41+42",       "(U,B)",             (1/7)*(2/3)*(1/3)*2),
  c("84,42+42",       "(B,B)",             (1/7)*(1/3)*(1/3))
  ),
  nrow=23, ncol=3, byrow=TRUE)





topol.to.index <- function(topol) {
  w <- grep(topol, eighttip.topologies[,1], fixed=TRUE)
  stopifnot(length(w) == 1)
  w[1]
}


gettopol.probs.for.ops <- function(ops) {
  trees <- read.trees.remove.burnin(paste0(fpathbaseQ, ops, ".txt"))
  inode.count <- trees[[1]]$Nnode
  stopifnot(inode.count == 7)
  counts <- rep(0,23)
  for (t in 1:length(trees)) {
    tree <- trees[[t]]
    po.edge <- reorder.phylo(tree, "postorder")$edge
    nodeinfo <- poedges.to.nodeinfo(po.edge, inode.count)
    topol <- nodeinfo.to.topology8(nodeinfo, 15, "")
    cat(topol)
    i <- topol.to.index(topol)
    counts[i] <- counts[i] + 1
    cat(" ", i, " #  ")
  }
  counts <- counts / sum(counts)
  counts
}


get.and.plot.topol.probs.for.ops <- function(ops) {
  counts <- gettopol.probs.for.ops(ops)
  par(mar=c(6, 4, 2, 2) + 0.1, cex.axis=0.7)
  barplot(counts, space=0, ylim=c(0,1.2*max(counts)),
          ylab="probability",
          names.arg=eighttip.topologies[,2], las=2)
  mtext(text=paste0("Operators ", ops), line=0)
  points((1:23)-0.5, as.double(eighttip.topologies[,3]), pch=18, cex=1.5)
}



get.and.plot.topol.probs.for.ops.article <- function(ops, label) {
  counts <- gettopol.probs.for.ops(ops)
  barplot(counts, space=0, ylim=c(0,.15),
          ylab="probability",
          main=label,
          names.arg=eighttip.topologies[,2], las=3)
  points((1:23)-0.5, as.double(eighttip.topologies[,3]), pch=18)
}




##########################################################################
##########################################################################

### DELIMITATION: CLUSTERINGS


fpathbaseP <- "C:/Users/Work/AAA/Programming/ProgramOutput/STACEY/P/Pg1t10/P4STSg1t10r1s42"

read.clusterings <- function(ops) {
  x <- read.table(paste0(fpathbaseP, ops, "-SDA.txt"), header=TRUE)
  x
}


prob.partition.shape <- function(x) {
  r <- length(x)
  n <- sum(x)
  s <- rep(0,n)
  for (i in 1:n) {
    w <- which(x == i)
    s[i] <- length(w)
  }
  z <- factorial(r)
  z <- z / choose(n-1, r-1)
  for (i in 1:n) {
    z <- z / factorial(s[i])
  }
  z
}


make.matrix.of.clusterings <- function() {
  clusterings <- matrix(0, nrow=22, ncol=8)
  
  clusterings[1,1] <- c(8)
  
  clusterings[2,1:2] <- c(7,1)
  clusterings[3,1:2] <- c(6,2)
  clusterings[4,1:2] <- c(5,3)
  clusterings[5,1:2] <- c(4,4)
  
  clusterings[6,1:3] <- c(6,1,1)
  clusterings[7,1:3] <- c(5,2,1)
  clusterings[8,1:3] <- c(4,3,1)
  clusterings[9,1:3] <- c(4,2,2)
  clusterings[10,1:3] <- c(3,3,2)
  
  clusterings[11,1:4] <- c(5,1,1,1)
  clusterings[12,1:4] <- c(4,2,1,1)
  clusterings[13,1:4] <- c(3,3,1,1)
  clusterings[14,1:4] <- c(3,2,2,1)
  clusterings[15,1:4] <- c(2,2,2,2)
  
  clusterings[16,1:5] <- c(4,1,1,1,1)
  clusterings[17,1:5] <- c(3,2,1,1,1)
  clusterings[18,1:5] <- c(2,2,2,1,1)
  
  clusterings[19,1:6] <- c(3,1,1,1,1,1)
  clusterings[20,1:6] <- c(2,2,1,1,1,1)
  
  clusterings[21,1:7] <- c(2,1,1,1,1,1,1)
  
  clusterings[22,1:8] <- c(1,1,1,1,1,1,1,1)
  
  rownames(clusterings) <- c("8", 
                             "7+1",         "6+2",       "5+3",     "4+4", 
                             "6+1+1",       "5+2+1",     "4+3+1",   "4+2+2",   "3+3+2",
                             "5+1+1+1",     "4+2+1+1",   "3+3+1+1", "3+2+2+1", "2+2+2+2", 
                             "4+1+1+1+1",   "3+2+1+1+1", "2+2+2+1+1", 
                             "3+1+1+1+1+1", "2+2+1+1+1+1", 
                             "2+1+1+1+1+1+1", 
                             "1+1+1+1+1+1+1+1")
  
  clusterings
}




getclusterings.probs.for.ops <- function(ops) {
  c.matrix <- make.matrix.of.clusterings()
  names <- rownames(c.matrix)
  
  counts <- rep(0, nrow(c.matrix))
  x <- read.clusterings(ops)
  for (i in 1:nrow(x)) {
    nc <- x[i,"nclusters"]
    ctr <- x[i,-(1:4)]
    sizecounts <- rep(0, nc)
    for (j in 1:nc) {
      sizecounts[j] <- length(which(ctr==j))
    }
    sizecounts <- sort(sizecounts, decreasing=TRUE)
    name <- ""
    for (j in 1:nc) {
      name <-  paste0(name, sizecounts[j])
      if (j < nc) {
        name <-  paste0(name, "+")
      }
    }
    m <- which(names==name)
    #cat(name, m, "   ")
    stopifnot(length(m) == 1)
    counts[m] <- counts[m] + x[i,"count"]
  }
  print(counts)
  counts / sum(counts)
}




get.and.plot.clusterings.probs.for.ops <- function(ops) {
  c.matrix <- make.matrix.of.clusterings()
  names <- rownames(c.matrix)
  probs <- rep(0, nrow(c.matrix))
  for (i in 1:nrow(c.matrix)) {
    x <- c.matrix[i,]
    probs[i] <- prob.partition.shape(x[x>0])
  }
  
  counts <- getclusterings.probs.for.ops(ops)
  par(mar=c(7, 4, 2, 0) + 0.1, cex.axis=0.7)
  barplot(counts, space=0, ylim=c(0,1.2*max(counts)),
          ylab="probability",
          names.arg=names, las=2)
  mtext(text=paste0("Operators ", ops), line=0)
  points((1:22)-0.5, probs/8, pch=18)
}


get.and.plot.clusterings.probs.for.ops.article <- function(ops, label) {
  c.matrix <- make.matrix.of.clusterings()
  names <- rownames(c.matrix)
  probs <- rep(0, nrow(c.matrix))
  for (i in 1:nrow(c.matrix)) {
    x <- c.matrix[i,]
    probs[i] <- prob.partition.shape(x[x>0])
  }
  counts <- getclusterings.probs.for.ops(ops)
  
  barplot(counts, space=0, ylim=c(0,.15),
          ylab="probability",
          main=label,
          names.arg=names, las=3)
  points((1:22)-0.5, probs/8, pch=18)
}



###############################################################################
###############################################################################
###############################################################################
###############################################################################


# PDFs TOPOLOGY
# pdf(file=paste0(fpathbaseQ, "topol.pdf"), width=7, height=7)
# par(mfcol=c(2,2))
# get.and.plot.topol.probs.for.ops("B")
# get.and.plot.topol.probs.for.ops("BF")
# get.and.plot.topol.probs.for.ops("BN")
# get.and.plot.topol.probs.for.ops("BR")
# dev.off()
# 
# # PDFs NODE HEIGHTS

get.and.make.pdf.node.heights.for.ops.kth.speciation <- function(ops, k) {
  pdf(file=paste0(fpathbaseQ, "heights", ops, "speciation", k, ".pdf"), width=4.2, height=5.4)
  get.and.plot.node.heights.for.ops.kth.speciation(ops, k)
  dev.off()
}

# get.and.make.pdf.node.heights.for.ops("B")
# get.and.make.pdf.node.heights.for.ops("BF")
# get.and.make.pdf.node.heights.for.ops("BN")
# get.and.make.pdf.node.heights.for.ops("BR")
# 
# # PDFs CLUSTERINGS
# pdf(file=paste0(fpathbaseP, "clustersizes.pdf"), width=7, height=7)
# par(mfcol=c(2,2))
# get.and.plot.clusterings.probs.for.ops("B")
# get.and.plot.clusterings.probs.for.ops("BF")
# get.and.plot.clusterings.probs.for.ops("BN")
# get.and.plot.clusterings.probs.for.ops("BR")
# dev.off()


# COMBINATION FOR ARTICLE

plot.for.article <- function() {
  oldpar <- par()
  layout(mat=matrix(c(1,2,3,4),nrow=2,ncol=2,byrow=TRUE), widths=c(1,1), heights=c(6,5))
  par(mar=c(7, 3, 2, 0) + 0.1, cex.axis=0.7, mgp=c(2,1,0))
  get.and.plot.clusterings.probs.for.ops.article("BR", "a")
  par(mar=c(7, 3, 2, 0) + 0.1)
  get.and.plot.topol.probs.for.ops.article("BR", "b")
  par(mar=c(5, 4, 2, 0) + 0.1)
  get.and.plot.node.heights.for.ops.kth.speciation.article("BR", 1, "c")
  get.and.plot.node.heights.for.ops.kth.speciation.article("BR", 7, "d")
  par(oldpar)
  
}

plot.for.article()


pdf(file=article.pdf, width=122/25.4, height=130/25.4, pointsize=10)
plot.for.article()
dev.off()

setEPS()
postscript(file=article.eps, width=122/25.4, height=130/25.4, pointsize=10)
plot.for.article()
dev.off()









